package api.common.util;

import java.util.*;
import java.util.function.Predicate;
import java.util.stream.Collectors;

public class CollectionUtil {

    // -------------------------------- A --------------------------------
    public static <T> List<T> arrayToList(T[] array) {
        return Arrays.asList(array);
    }

    // -------------------------------- B --------------------------------
    // -------------------------------- C --------------------------------
    @SafeVarargs
    public static <T> ArrayList<T> createArrayList(T... elements) {
        return new ArrayList<>(Arrays.asList(elements));
    }

    // -------------------------------- D --------------------------------
    // -------------------------------- F --------------------------------

    /**
     * 根据过滤条件来过滤列表
     *
     * @param arrayList 原始列表
     * @param predicate 过滤条件的 lambda 表达式，a-> a.getColor().equals("red") && a.getWeight() > 300
     * @param <T>       声明为泛型方法
     * @return 结果列表
     */
    public static <T> List<T> filterList(List<T> arrayList, Predicate<? super T> predicate) {
        return arrayList.stream().filter(predicate).collect(Collectors.toList());
    }

    // -------------------------------- G --------------------------------
    // -------------------------------- H --------------------------------
    // -------------------------------- I --------------------------------
    public static boolean isEmpty(Collection<?> collection) {
        return collection == null || collection.isEmpty();
    }

    public static boolean isEmpty(Map<?, ?> map) {
        return map == null || map.isEmpty();
    }

    public static boolean isNotEmpty(Collection<?> collection) {
        return !isEmpty(collection);
    }

    public static boolean isNotOne(Collection<?> collection) {
        return isEmpty(collection) || collection.size() > 1;
    }

    public static boolean isNotEmpty(Map<?, ?> map) {
        return !isEmpty(map);
    }

    // -------------------------------- J --------------------------------
    // -------------------------------- K --------------------------------
    // -------------------------------- L --------------------------------
    // -------------------------------- M --------------------------------
    // -------------------------------- N --------------------------------
    // -------------------------------- O --------------------------------
    // -------------------------------- P --------------------------------
    // -------------------------------- Q --------------------------------
    // -------------------------------- R --------------------------------
    // -------------------------------- S --------------------------------
    // -------------------------------- T --------------------------------
    // -------------------------------- U --------------------------------
    // -------------------------------- V --------------------------------
    // -------------------------------- W --------------------------------
    // -------------------------------- X --------------------------------
    // -------------------------------- Y --------------------------------
    // -------------------------------- Z --------------------------------


}
