package api.common.util;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.boot.configurationprocessor.json.JSONArray;
import org.springframework.boot.configurationprocessor.json.JSONException;
import org.springframework.boot.configurationprocessor.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

public class JsonUtil {

    // -------------------------------- A --------------------------------
    // -------------------------------- B --------------------------------

    /**
     * bean 转成 json
     *
     * @param bean bean 对象
     * @param <T>  声明为泛型方法
     * @return json 字符串
     * @throws JsonProcessingException 异常
     */
    public static <T> String beanToJson(T bean) throws JsonProcessingException {
        return new ObjectMapper().writeValueAsString(bean);
    }
    // -------------------------------- C --------------------------------
    // -------------------------------- D --------------------------------
    // -------------------------------- F --------------------------------
    // -------------------------------- G --------------------------------
    // -------------------------------- H --------------------------------
    // -------------------------------- I --------------------------------
    // -------------------------------- J --------------------------------

    /**
     * json 转成 JSONObject
     *
     * @param json json 字符串
     * @return JSONObject 对象
     * @throws JSONException 异常
     */
    public static JSONObject jsonToJSONObject(String json) throws JSONException {
        return new JSONObject(json);
    }

    /**
     * json 转成 bean
     *
     * @param json   json 字符串
     * @param tClass 返回值类型
     * @param <T>    声明为泛型方法
     * @return bean 对象
     * @throws JsonProcessingException 异常
     */
    public static <T> T jsonToBean(String json, Class<T> tClass) throws JsonProcessingException {
        return new ObjectMapper().readValue(json, tClass);
    }

    /**
     * JSONObject 转成 bean
     *
     * @param jsonObject   jsonObject 对象
     * @param tClass 返回值类型
     * @param <T>    声明为泛型方法
     * @return bean 对象
     */
    public static <T> T jsonObjectToBean(JSONObject jsonObject, Class<T> tClass) throws JsonProcessingException {
        return new ObjectMapper().readValue(jsonObject.toString(), tClass);
    }

    /**
     * json 转成 list
     *
     * @param arrayJson    json 数组字符串
     * @param elementClass 列表元素类型
     * @param <E>          声明为泛型方法
     * @return 结果列表
     * @throws JsonProcessingException 异常
     */
    public static <E> List<E> jsonToList(String arrayJson, Class<E> elementClass) throws JsonProcessingException, JSONException {
        List<E> resultList = new ArrayList<>();
        JSONArray jsonArray = new JSONArray(arrayJson);
        for (int i = 0; i < jsonArray.length(); i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);
            String beanJson = jsonObject.toString();
            E element = jsonToBean(beanJson, elementClass);
            resultList.add(element);
        }
        return resultList;
    }

//    /**
//     * json 字符串转成 info 为 list 的 result
//     *
//     * @param json         json 字符串
//     * @param elementClass 列表元素类型
//     * @param <E>          声明为泛型方法
//     * @return result 对象
//     * @throws JsonProcessingException 异常
//     */
//    public static <E> Result<List<E>> jsonToResultWithListInfo(String json, Class<E> elementClass) throws JsonProcessingException, JSONException {
//        Result<?> temporaryResult = jsonToBean(json, Result.class);
//        String infoJson = beanToJson(temporaryResult.getInfo());
//        List<E> infoList = jsonToList(infoJson, elementClass);
//        Result<List<E>> finalResult = new Result<>();
//        finalResult.setCode(temporaryResult.getCode());
//        finalResult.setMessage(temporaryResult.getMessage());
//        finalResult.setInfo(infoList);
//        return finalResult;
//    }
//
//    /**
//     * json 字符串转成 info 为 list 的 result
//     *
//     * @param jsonObject   JSONObject 对象
//     * @param elementClass 列表元素类型
//     * @param <E>          声明为泛型方法
//     * @return result 对象
//     * @throws JsonProcessingException 异常
//     */
//    public static <E> Result<List<E>> jsonObjectToResultWithListInfo(JSONObject jsonObject, Class<E> elementClass) throws JsonProcessingException, JSONException {
//        String json = jsonObject.toString();
//        return jsonToResultWithListInfo(json, elementClass);
//    }
    // -------------------------------- K --------------------------------
    // -------------------------------- L --------------------------------

    /**
     * bean 转成 json
     *
     * @param list 列表对象
     * @param <T>  声明为泛型方法
     * @return json 字符串
     * @throws JsonProcessingException 异常
     */
    public static <T> String listToJson(T list) throws JsonProcessingException {
        return new ObjectMapper().writeValueAsString(list);
    }

    // -------------------------------- M --------------------------------
    // -------------------------------- N --------------------------------
    // -------------------------------- O --------------------------------
    // -------------------------------- P --------------------------------
    // -------------------------------- Q --------------------------------
    // -------------------------------- R --------------------------------
    // -------------------------------- S --------------------------------
    // -------------------------------- T --------------------------------
    // -------------------------------- U --------------------------------
    // -------------------------------- V --------------------------------
    // -------------------------------- W --------------------------------
    // -------------------------------- X --------------------------------
    // -------------------------------- Y --------------------------------
    // -------------------------------- Z --------------------------------

}
