package com.css.simulation.resource.scheduler.scheduler;


import api.common.util.JsonUtil;
import com.css.simulation.resource.scheduler.mapper.JobMapper;
import com.css.simulation.resource.scheduler.mapper.TaskMapper;
import com.css.simulation.resource.scheduler.pojo.dto.task.*;
import com.css.simulation.resource.scheduler.pojo.po.JobPO;
import com.css.simulation.resource.scheduler.pojo.po.TaskPO;
import com.fasterxml.jackson.core.JsonProcessingException;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@Component
public class JobScheduler {

    @Resource
    JobMapper jobMapper;
    @Resource
    TaskMapper taskMapper;
    @Resource
    private KafkaTemplate<String, String> kafkaTemplate;

    @Scheduled(cron = "*/5 * * * *")
    public void parseJob() throws JsonProcessingException {
        //1 读取 mysql 获取待执行的且创建时间最早的 job 信息
        JobPO jobPO = jobMapper.selectWaitingJob();
        String jobId = jobPO.getId();                   // 工作id
        String defaultTime = jobPO.getDefaultTime();    // 任务执行时间，一个工作里的所有任务公用
        String parallelism = jobPO.getParallelism();    // 并行度，创建 pod 时使用
        //2 修改该 job 的状态为执行中，防止下一个定时读取到
        jobMapper.updateJobState(jobId, "2");
        //3 根据 job 信息获取 task 信息。task 信息包括算法、模型等
        List<TaskPO> taskPOList = taskMapper.selectByJobId(jobId);
        //4 组装 task 消息
        for (TaskPO taskPO : taskPOList) {
            String taskId = taskPO.getId();
            //4-1 初始化所有对象
            TaskDTO taskDTO = new TaskDTO();

            InfoDTO infoDTO = new InfoDTO();
            ScenarioDTO scenarioDTO = new ScenarioDTO();

            VehicleDTO vehicleDTO = new VehicleDTO();
            ModelDTO modelDTO = new ModelDTO();
            DynamicsDTO dynamicsDTO = new DynamicsDTO();
            SensorsDTO sensorsDTO = new SensorsDTO();

            List<CameraDTO> cameraDTOList = new ArrayList<>();
            List<OgtDTO> ogtDTOList = new ArrayList<>();

            //4-2 将所有初始化对象赋值

            //4-3 组装
            sensorsDTO.setCamera(cameraDTOList);
            sensorsDTO.setOGT(ogtDTOList);

            vehicleDTO.setModel(modelDTO);
            vehicleDTO.setDynamics(dynamicsDTO);
            vehicleDTO.setSensors(sensorsDTO);

            taskDTO.setInfo(infoDTO);
            taskDTO.setScenario(scenarioDTO);
            taskDTO.setVehicle(vehicleDTO);

            //4-4 将对象转成 json
            String taskJson = JsonUtil.beanToJson(taskDTO);

            //4-5 发送到 kafka
            kafkaTemplate.send(jobId, 0, taskId, taskJson);

        }
        //3 将 job 名称作为 topic 名称，发送 task 信息到 kafka
        //4 创建 pod 开始执行。
    }
}
